unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqlite3conn, sqldb, db, FileUtil, Forms, Controls,
  Graphics, Dialogs, DbCtrls, DBGrids, Grids;

type

  { TForm1 }

  TForm1 = class(TForm)
    DBMemo1: TDBMemo;
    DsSales: TDataSource;
    DsArticles: TDataSource;
    DBGrid1: TDBGrid;
    DBNavigator1: TDBNavigator;
    SQLite3Connection1: TSQLite3Connection;
    QrySales: TSQLQuery;
    QryArticles: TSQLQuery;
    SQLTransaction1: TSQLTransaction;
    procedure DBGrid1DrawColumnCell(Sender: TObject; const Rect: TRect;
      DataCol: Integer; Column: TColumn; State: TGridDrawState);
    procedure FormCreate(Sender: TObject);
  private
    procedure CreateTables;
    procedure PopulateTables;

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  LCLType, LCLIntf, LazUTF8;

const
  DB_NAME = 'test.sqlite';

type
  TMyDBGrid = class(TDBGrid);

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
  h: Integer;
begin
  SQLite3Connection1.DatabaseName := DB_NAME;
  DBMemo1.DataField := 'Comments';
  if not FileExists(DB_NAME) then begin
    CreateTables;
    PopulateTables;
  end;
  SQLite3Connection1.Connected := true;
  QryArticles.SQL.Add('SELECT * FROM Articles');
  QryArticles.Open;
  QrySales.SQL.Add('SELECT * FROM Sales');
  QrySales.Open;

  h := DBGrid1.DefaultRowHeight;
  // Zeilenhöhe für 3 Textzeilen
  DBGrid1.DefaultRowHeight := DBGrid1.Canvas.TextHeight('A') * 3 + 2*constCellPadding;;
  TMyDBGrid(DBGrid1).RowHeights[0] := h;

  // Breite der Artikel-Spalte
  DBGrid1.Columns[1].Width := 100;

  // Breite der Memo-Spalte
  for i:=0 to DBGrid1.Columns.Count-1 do
    if DBGrid1.Columns[i].Field.DataType = ftMemo then
      DBGrid1.Columns[i].Width := 200;
end;

procedure TForm1.CreateTables;
begin
  SQLTransaction1.StartTransaction;
  SQLite3Connection1.Connected := true;
  SQLite3Connection1.ExecuteDirect('CREATE TABLE Articles (' +
    'ArticleID INTEGER UNIQUE NOT NULL, '+
    'ArticleName VARCHAR(50), '+
    'Comments TEXT, '+
    'Price FLOAT)');
  SQLite3Connection1.ExecuteDirect('CREATE TABLE Sales ('+
    'ArticleID INTEGER NOT NULL, '+
    'PiecesSold INTEGER NOT NULL, '+
    'DateSold DATE NOT NULL)');
  SQLTransaction1.Commit;
  SQLite3Connection1.Connected := false;
end;

procedure TForm1.DBGrid1DrawColumnCell(Sender: TObject; const Rect: TRect;
  DataCol: Integer; Column: TColumn; State: TGridDrawState);
var
  field: TField;
  ts: TTextStyle;
  s: String;
  h, hRect: Integer;
  i, dn, n, nprev: Integer;
  R: TRect;
  p: PChar;
begin
  field := DBGrid1.Columns[DataCol].Field;
  if field.DataType = ftMemo then begin
    DBGrid1.Canvas.FillRect(Rect);
    ts := DBGrid1.Canvas.TextStyle;
    ts.WordBreak := true;
    ts.SingleLine := false;
    DBGrid1.Canvas.TextStyle := ts;
    h := 0;
    hRect := Rect.Bottom - Rect.Top;
    s := field.AsString;
    n := 0;
    nprev := 0;
    p := @s[1];
    for i:=1 to Length(s) do begin
      dn := UTF8CharacterLength(p);
      n := n + dn;
      R := Rect;
      InflateRect(R, -constCellPadding, -constCellPadding);
      DrawText(DBGrid1.Canvas.Handle, PChar(s), n, R, DT_CALCRECT + DT_WORDBREAK);
      h := R.Bottom - R.Top + 2*constCellPadding;
      if h > hRect then begin
        n := nprev;
        break;
      end;
      nprev := n;
      inc(p, dn);
    end;
    DBGrid1.Canvas.FillRect(Rect);
    R := Rect;
    InflateRect(R, -constCellPadding, -constCellPadding);
    DrawText(DBGrid1.Canvas.Handle, PChar(s), n, R, DT_WORDBREAK + DT_LEFT + DT_VCENTER);
  end;
end;

procedure TForm1.PopulateTables;
const
  N = 5;
var
  i, j: Integer;
  id: Integer;
  m: Integer;
  s: String;
  sc: String;
  p: Double;
  fs: TFormatSettings;
  dt: TDate;
begin
  SQLTransaction1.StartTransaction;
  fs := FormatSettings;
  fs.DecimalSeparator := '.';
  for i:=0 to N-1 do begin
    id := Random(100000);
    p := Random * 1000;
    SetLength(s, 3 + Random(5));
    for j:=1 to Length(s) do
      s[j] := char(Random(26) + ord('A'));
    sc := IntToStr(i) + LineEnding +
      'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.';
    SQLite3Connection1.ExecuteDirect(Format(
      'INSERT INTO Articles (ArticleID, ArticleName, Comments, Price) VALUES (%d, "%s", "%s", %.2f)', [
      id, s, sc, p
    ], fs));
    for j:=0 to 10*n-1 do begin
      m := random(10) + 1;
      dt := SysUtils.Date() - 1000 + Random * 1000;
      SQLite3Connection1.ExecuteDirect(Format(
        'INSERT INTO Sales (ArticleID, PiecesSold, DateSold) VALUES (%d, %d, %s)',
        [id, m, FormatDateTime('yyyymmdd', dt)
      ]));;
    end;
  end;
  SQLTransaction1.Commit;
end;

end.

